﻿using Newtonsoft.Json;
using System.Net.Http.Headers;
using System.Text;

namespace WebApp;

public class APIClient
{
    private static readonly HttpClient _client = new();

    public static void Connect(IConfiguration configuration)
    {
        _client.BaseAddress = new Uri(configuration["API"]);
        _client.DefaultRequestHeaders.Accept.Clear();
        _client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
    }

    public static T? GetRequest<T>(string requestUrl)
    {
        var response = _client.GetAsync(requestUrl);
        var result = response.Result.Content.ReadAsStringAsync().Result;
        if (!response.Result.IsSuccessStatusCode)
        {
            throw new Exception(response.Result.ReasonPhrase);
        }
        return JsonConvert.DeserializeObject<T>(result);
    }

    public static object? PostRequest<T>(string requestUrl, T model)
    {
        var json = JsonConvert.SerializeObject(model);
        var data = new StringContent(json, Encoding.UTF8, "application/json");

        var response = _client.PostAsync(requestUrl, data);

        var result = response.Result.Content.ReadAsStringAsync().Result;
        if (!response.Result.IsSuccessStatusCode)
        {
            throw new Exception(response.Result.ReasonPhrase);
        }
        return result;
    }

    public static object? DeleteRequest(string requestUrl)
    {
        var response = _client.DeleteAsync(requestUrl);

        var result = response.Result.Content.ReadAsStringAsync().Result;
        if (!response.Result.IsSuccessStatusCode)
        {
            throw new Exception(response.Result.ReasonPhrase);
        }
        return result;
    }

    public static object? PatchRequest<T>(string requestUrl, T model)
    {
        var json = JsonConvert.SerializeObject(model);
        var data = new StringContent(json, Encoding.UTF8, "application/json");

        var response = _client.PatchAsync(requestUrl, data);

        var result = response.Result.Content.ReadAsStringAsync().Result;
        if (!response.Result.IsSuccessStatusCode)
        {
            throw new Exception(response.Result.ReasonPhrase);
        }
        return result;
    }
}